@@ -23,6 +23,8 @@
 
 #include "config.h"
 
+#include <X11/Xlib.h>
+#include <X11/Xatom.h>
 #include <string.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -72,7 +74,7 @@ static guint atk_signal_link_selected;
 static guint atk_signal_bounds_changed;
 
 static Accessibility_Registry spi_atk_bridge_get_registry (void);
-static void     spi_atk_bridge_do_registration         (void);
+static gboolean spi_atk_bridge_do_registration         (void);
 static void     spi_atk_bridge_toplevel_added          (AtkObject             *object,
                                                         guint                 index,
                                                         AtkObject             *child);
@@ -83,6 +85,7 @@ static void     spi_atk_bridge_toplevel_removed        (AtkObject             *o
 static void     spi_atk_bridge_exit_func               (void);
 static void     spi_atk_register_event_listeners       (void);
 static void     spi_atk_bridge_focus_tracker           (AtkObject             *object);
+static gchar   *spi_atk_bridge_get_registry_ior        (void);
 static void     spi_atk_bridge_register_application    (Accessibility_Registry registry);
 static gboolean spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
 							guint                  n_param_values,
@@ -107,6 +110,11 @@ spi_atk_bridge_window_event_listener (GSignalInvocationHint *signal_hint,
 				const GValue *param_values,
 				gpointer data);
 static gboolean
+spi_atk_bridge_document_event_listener (GSignalInvocationHint *signal_hint,
+                                guint n_param_values,
+                                const GValue *param_values,
+                                gpointer data);
+static gboolean
 spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
 				     guint n_param_values,
 				     const GValue *param_values,
@@ -167,6 +175,7 @@ atk_bridge_init (gint *argc, gchar **argv[])
 {
   const char *debug_env_string = g_getenv ("AT_SPI_DEBUG");
   gchar *fname;
+  gboolean success = FALSE;
 
   if (atk_bridge_initialized)
     {
@@ -214,16 +223,22 @@ atk_bridge_init (gint *argc, gchar **argv[])
     }
   else
     {
-      spi_atk_bridge_do_registration ();
+      success = spi_atk_bridge_do_registration ();
+    }
+  if (success) 
+    {
+      spi_atk_register_event_listeners ();
+      spi_atk_bridge_init_event_type_consts ();
+    }
+  else
+    {
+      atk_bridge_initialized = FALSE;
     }
-  spi_atk_register_event_listeners ();
-  spi_atk_bridge_init_event_type_consts ();
- 
   return 0;
 }
 
 
-static void
+static gboolean
 spi_atk_bridge_do_registration (void)
 {
   CORBA_Environment ev;
@@ -233,6 +248,7 @@ spi_atk_bridge_do_registration (void)
   if (spi_atk_bridge_get_registry () == CORBA_OBJECT_NIL)
     {
       g_error ("Could not locate registry");
+      return FALSE;
     }
 
   bonobo_activate ();
@@ -248,7 +264,7 @@ spi_atk_bridge_do_registration (void)
   g_atexit (spi_atk_bridge_exit_func);
 
   DBG (1, g_message ("Application registered & listening\n"));
-
+  return TRUE;
 }
 
 static void
@@ -327,10 +343,41 @@ spi_display_name (void)
     return canonical_display_name;
 }
 
+static     Display *bridge_display = NULL;
+
+static gchar *
+spi_atk_bridge_get_registry_ior (void)
+{
+     
+     Atom AT_SPI_IOR;
+     Atom actual_type;
+     int actual_format;
+     unsigned char *data = NULL;  
+     unsigned long nitems;
+     unsigned long leftover;
+     if (!bridge_display) 
+       bridge_display = XOpenDisplay (spi_display_name ());
+
+     AT_SPI_IOR = XInternAtom (bridge_display, "AT_SPI_IOR", False); 
+     XGetWindowProperty(bridge_display, 
+			XDefaultRootWindow (bridge_display),
+			AT_SPI_IOR, 0L, 
+			(long)BUFSIZ, False, 
+			(Atom) 31, &actual_type, &actual_format,
+			&nitems, &leftover, &data);
+     if (data == NULL)
+	  g_critical (_("AT_SPI_REGISTRY was not started at session startup."));
+     
+     return (gchar *) data;
+     
+}
+
+
 static Accessibility_Registry
 spi_atk_bridge_get_registry (void)
 {
   CORBA_Environment ev;
+  char *ior =  NULL;
 
   if (registry_died || (registry == CORBA_OBJECT_NIL)) {
 	  CORBA_exception_init (&ev);
@@ -341,10 +388,17 @@ spi_atk_bridge_get_registry (void)
               else
 	        DBG (1, g_warning ("registry died! restarting..."));
             }
-	  bonobo_activation_set_activation_env_value ("AT_SPI_DISPLAY", spi_display_name ());
 
-	  registry = bonobo_activation_activate_from_id (
-		  "OAFIID:Accessibility_Registry:1.0", 0, NULL, &ev);
+	  /* XXX: This presumes that the registry has successfully restarted itself already...*/
+	  ior = (char *) spi_atk_bridge_get_registry_ior ();
+
+	  if (ior != NULL)
+	       registry = CORBA_ORB_string_to_object (bonobo_activation_orb_get (), 
+						      ior, &ev);
+	  else {
+	       g_warning ("IOR not set.");  
+	       registry = CORBA_OBJECT_NIL;
+	  }
 	  
 	  if (ev._major != CORBA_NO_EXCEPTION)
 	  {
@@ -453,6 +507,15 @@ spi_atk_register_event_listeners (void)
   id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
 				      "window:deactivate");
   g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_document_event_listener,
+				      "Gtk:AtkDocument:load-complete");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_document_event_listener,
+				      "Gtk:AtkDocument:reload");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_document_event_listener,
+				      "Gtk:AtkDocument:load-stopped");
+  g_array_append_val (listener_ids, id);
   id = atk_add_global_event_listener (spi_atk_bridge_state_event_listener,
 				      "Gtk:AtkObject:state-change");
   g_array_append_val (listener_ids, id);
@@ -541,6 +604,8 @@ spi_atk_bridge_exit_func (void)
     {
       g_assert (!bonobo_debug_shutdown ());
     }
+  if (bridge_display)
+    XCloseDisplay (bridge_display);
 }
 
 void
@@ -574,14 +639,16 @@ gnome_accessibility_module_shutdown (void)
     }
 
   listener_ids = NULL;
-  atk_remove_focus_tracker (atk_bridge_focus_tracker_id);
+  if (atk_bridge_focus_tracker_id)
+        atk_remove_focus_tracker (atk_bridge_focus_tracker_id);
   
   for (i = 0; ids && i < ids->len; i++)
   {
           atk_remove_global_event_listener (g_array_index (ids, guint, i));
   }
   
-  atk_remove_key_event_listener (atk_bridge_key_event_listener_id);
+  if (atk_bridge_key_event_listener_id)
+          atk_remove_key_event_listener (atk_bridge_key_event_listener_id);
 
   deregister_application (app);
 }
@@ -1032,6 +1099,7 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
     }
   else if ((signal_query.signal_id == atk_signal_children_changed) && gobject)
     {
+      detail1 = g_value_get_uint (param_values + 1);
       ao = atk_object_ref_accessible_child (ATK_OBJECT (gobject), 
 			        	    detail1);
       if (ao) 
@@ -1126,6 +1194,41 @@ spi_atk_bridge_window_event_listener (GSignalInvocationHint *signal_hint,
   return TRUE;
 }
 
+static gboolean
+spi_atk_bridge_document_event_listener (GSignalInvocationHint *signal_hint,
+                                      guint n_param_values,
+                                      const GValue *param_values,
+                                      gpointer data)
+{
+  GObject *gobject;
+  GSignalQuery signal_query;
+  CORBA_any any;
+  const gchar *name, *s;
+#ifdef SPI_BRIDGE_DEBUG
+  const gchar *s2;
+#endif
+
+  g_signal_query (signal_hint->signal_id, &signal_query);
+
+  name = signal_query.signal_name;
+
+#ifdef SPI_BRIDGE_DEBUG
+  s2 = g_type_name (G_OBJECT_TYPE (g_value_get_object (param_values + 0)));
+  s = atk_object_get_name (ATK_OBJECT (g_value_get_object (param_values + 0)));
+  fprintf (stderr, "Received signal %s:%s from object %s (gail %s)\n",
+           g_type_name (signal_query.itype), name, s ? s : "<NULL>" , s2);
+#endif
+
+  gobject = g_value_get_object (param_values + 0);
+
+  s = atk_object_get_name (ATK_OBJECT (gobject));
+  spi_atk_bridge_init_string (&any, ATK_OBJECT (gobject), (gchar **) &s);
+
+  spi_atk_emit_eventv (gobject, 0, 0, &any,
+                       "document:%s", name);
+  return TRUE;
+}
+
 static void
 spi_atk_tidy_windows (void)
 {
