@@ -25,19 +25,25 @@
 #include <stdlib.h>
 #endif
 
-#include <gdk/gdk.h>
+#include <config.h>
+#include <string.h>
+#include <gdk/gdkx.h>
 #include <libbonobo.h>
 #include <glib/gmain.h>
 #include "registry.h"
 
+#define spi_get_display() GDK_DISPLAY()
+
+static void registry_set_ior (SpiRegistry *registry);
+
 int
 main (int argc, char **argv)
 {
   int          ret;
-  char        *obj_id, *display_name;
+  char        *obj_id;
+  const char  *display_name;
   char        *cp, *dp;
   SpiRegistry *registry;
-  GSList      *reg_env = NULL;
 
   if (!bonobo_init (&argc, argv))
     {
@@ -51,19 +57,18 @@ main (int argc, char **argv)
   display_name = g_getenv ("AT_SPI_DISPLAY");
   if (!display_name)
   {
-      display_name = g_strdup (gdk_display_get_name (gdk_display_get_default ()));
+      display_name = g_getenv ("DISPLAY");
       cp = strrchr (display_name, '.');
       dp = strrchr (display_name, ':');
-      if (cp && dp && ((guint) cp > (guint) dp)) *cp = '\0';
+      if (cp && dp && (cp > dp)) *cp = '\0';
   }
 
-  reg_env = bonobo_activation_registration_env_set ( reg_env, "AT_SPI_DISPLAY", 
-						     display_name);
   ret = bonobo_activation_register_active_server (
 	  obj_id,
 	  bonobo_object_corba_objref (bonobo_object (registry)),
-	  reg_env);
-  bonobo_activation_registration_env_free (reg_env);
+	  NULL);
+
+  registry_set_ior (registry);
 
   if (ret != Bonobo_ACTIVATION_REG_SUCCESS)
     {
@@ -82,3 +87,32 @@ main (int argc, char **argv)
   return 0;
 }
 
+static void
+registry_set_ior (SpiRegistry *registry){
+  CORBA_Environment ev;
+  Atom  AT_SPI_IOR = XInternAtom (spi_get_display (), "AT_SPI_IOR", FALSE);
+  char *iorstring = NULL;
+
+  CORBA_exception_init (&ev);
+
+  iorstring = CORBA_ORB_object_to_string (bonobo_activation_orb_get (),
+                                     bonobo_object_corba_objref (bonobo_object (registry)),
+                                     &ev);
+
+  XChangeProperty (spi_get_display(),
+		   XDefaultRootWindow (spi_get_display ()),
+		   AT_SPI_IOR, (Atom) 31, 8,
+		   PropModeReplace,
+		   (unsigned char *) iorstring,
+		   iorstring ? strlen (iorstring) : 0);
+
+  if (ev._major != CORBA_NO_EXCEPTION)
+	  {
+		  g_error ("Error setting IOR %s",
+			   CORBA_exception_id (&ev));
+                  CORBA_exception_free (&ev);
+           }
+
+  CORBA_exception_free (&ev);
+
+}
